import java.util.regex.Matcher;
import java.util.regex.Pattern;



public class Checker {

	private FlatFile flatFile;
	private Log log;

	public Checker(){
	}
	
	public void restartChecker(FlatFile flatFile, String logFileName){
		log = new Log(logFileName, flatFile.fileName);
		this.flatFile = flatFile;		
	}		
	
	public void startLogging(String columnDelimiter, int numberOfColumns, 
		String textQualifier){
		log.openFile();
		log.writeHeader(columnDelimiter, numberOfColumns, textQualifier);		
	}
	
	public void stopLogging(){
		log.writeFooter(flatFile.getNumberOfExceptions());
		log.closeFile();
	}
	
	private String[] breakInColumns(String line){
		String[] s = line.split(Pattern.quote(flatFile.columnDelimiter));
		return s;
	}
	
	private boolean isColumnCountOk(String line){
		boolean result = false; 	
		String columnDelimiters = line.replaceAll("[^"+Pattern.quote(flatFile.columnDelimiter)+"]", "");		
		if((columnDelimiters.length() + 1) == flatFile.numberOfColumns)
			result = true;		
		return result;
	}
	
	private boolean isWellFormedString(String column){
		boolean result = false;
		if(column.startsWith(flatFile.textQualifier) && column.endsWith(flatFile.textQualifier))
			result = true;
		else if(!column.contains(flatFile.textQualifier))
			result = true;
		return result;
	}
	
	public boolean isTextQualifierOk(String [] columns){
		boolean result = true;
		if(flatFile.textQualifier.isEmpty()){
			result = true;
		}
		else{
			for(int i = 0; i < columns.length; i++){
				if(!isWellFormedString(columns[i].trim())){
					result = false;
					break;					
				}
			}
		}
		return result;
	}	
	
	public boolean isFlatFileOk(){
		boolean result = true;
		String line;
		String [] columns;
		int rowNumber; 
		
		for(rowNumber = 1; true; rowNumber++){
			line = flatFile.getLine();
			if(line.compareTo("") == 0)
				break;
			
			if(!isColumnCountOk(line)){
				log.write(1, rowNumber);
				result = false;			
			}
			
			columns = breakInColumns(line);						
			if(!isTextQualifierOk(columns)){				
				result = false;
				log.write(2, rowNumber);
			}
		}
		return result;
	}
	
	public boolean checkFlatFile(){
		boolean result;
		startLogging(flatFile.columnDelimiter, flatFile.numberOfColumns, flatFile.textQualifier);
		System.out.print("\nOpening file "+flatFile.fileName+"... ");
		flatFile.openFile();
		System.out.print("Done.\nChecking file ...");
		result = isFlatFileOk();
		System.out.print("Done.\n");		
		flatFile.closeFile();	
		stopLogging();
		return result;
	}
	
	public int getNumberOfWarnings(){
		return log.getNumberOfWarnings();
	}
	
	public int getNumberOfExceptions(){
		return flatFile.getNumberOfExceptions();
	}
	
}
