import java.io.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class Controller {
	private BufferedReader input;
	private String metadataName;
	private GlobalLog globalLog;
	private Checker checker;
	
	private String center;
	private int numberOfFlatFiles;
	
	private String flatFileName;
	private int numberOfColumns;
	private String columnSeparator;
	private String rowSeparator;
	private String textQualifier;
	private String logFolder;
		
	private final int INITIAL_NUMBER_OF_FLAT_FILES = 15;
	private final int SIZE_OF_HEADER = 5;
	private final int SIZE_OF_BLOC = 5;
	private ArrayList<FlatFile> flatFiles;
	
	private String newLogFolder;
	
	public Controller(String metadataFileName){
		this.metadataName = metadataFileName;
		flatFiles = new ArrayList<FlatFile>(INITIAL_NUMBER_OF_FLAT_FILES);
		checker = new Checker();
		
	}
	
	boolean createDir(String dirName){
		boolean result = false;		
		result = new File(dirName).mkdir();		
		return result; 
	}
	
	
	void startGlobalLog(String date, String center){
		SimpleDateFormat s = new SimpleDateFormat("yyyyMMddHHmmss"); 
		String dateTime = s.format(Calendar.getInstance().getTime());
		
		newLogFolder = logFolder+"\\LOG_"+center+"_"+date.replace('/', '_')+"_"+"flat_files_"+dateTime;
		
		if(createDir(newLogFolder)){			
			globalLog = new GlobalLog(newLogFolder+"\\LOG_"+center+"_"+date.replace('/', '_'));
			globalLog.openFile();		
			globalLog.writeHeader(center, numberOfFlatFiles);
		}
	}
	
	public String stripPathFromName(String fileName){
		String result = null;
		Pattern pattern = Pattern.compile(".*[\\\\/](.*)$");
		Matcher matcher = pattern.matcher(fileName);
		if(matcher.matches()){
			result = matcher.group(1);
		}
		return result;
	}
	
	public boolean verifyFlatFiles(String date, String center){
		boolean result = true;
		FlatFile ff;
		Iterator<FlatFile> i;
		
		openMetadataFile();
		getMetadata();
		closeMetadadaFile();
		
		startGlobalLog(date, center);
						
		if(flatFiles.size() != numberOfFlatFiles)
			globalLog.write(3);
		
		i = flatFiles.iterator();		
		while(i.hasNext()){
			ff = (FlatFile)i.next();
			checker.restartChecker(ff, this.newLogFolder+"\\"+stripPathFromName(ff.fileName)+"_LOG");
			result = checker.checkFlatFile();
			globalLog.writeFlatFileSumary(ff.fileName, checker.getNumberOfWarnings(), checker.getNumberOfExceptions());
		}
		//globalLog.writeFooter();
		globalLog.closeFile();		
		System.out.println("\nFinished.");
		return result;
	}
	
	public void closeMetadadaFile(){
		try {
			input.close();
		} catch (IOException e) {
			System.err.println(e);
		}
	}
	
	public void openMetadataFile(){
		try{
			File file = new File(metadataName);
			input = new BufferedReader(new FileReader(file));
		}
		catch(Exception e){
			System.err.println(e);
		}	
	}

	public void instantiateFlatFiles(){
		FlatFile ff = new FlatFile(flatFileName, rowSeparator, columnSeparator, numberOfColumns, textQualifier);
		flatFiles.add(ff);
	}
	
	public void getMetadata(){				
		getHeader();					
		while(getFlatFileMetadata()){				
			instantiateFlatFiles();
			restartAttributes();
		}		

	}
	
	public void restartAttributes(){
		flatFileName = "";
		rowSeparator = "";
		columnSeparator = "";
		numberOfColumns = -1;
		textQualifier = "";
	}
	
	
	public boolean parseLineBloc(int numberOfLines){
		String line;
		int i = 0;
		boolean result = true;
		
		try{			
			while(i < numberOfLines){
				line = input.readLine();
				if(line == null){					
					result = false;
					break;
				}
				if(line.compareTo("") != 0){
					parseLine(line);
					i++;
				}
			}
		}catch(IOException e){
			System.err.println(e);
			result = false;
		}			
		return result;
	}
	
	public boolean getFlatFileMetadata(){		
		return parseLineBloc(SIZE_OF_BLOC);
	}
	
	public void getHeader(){
		parseLineBloc(SIZE_OF_HEADER);

	}
	

	public void parseLine(String line){
		String[] args;
		args = line.split("=");
		args[0] = args[0].trim();
		args[1] = args[1].trim();
		
		if(args[0].compareTo("center") == 0){
			this.center = args[1]; 
		}
		else if(args[0].compareTo("initial_date") == 0){
			//do nothing..
		}
		else if(args[0].compareTo("final_date") == 0){
			//do nothing..
		}
		else if(args[0].compareTo("number_of_flat_files") == 0){
			this.numberOfFlatFiles = Integer.parseInt(args[1]);
		}
		else if(args[0].compareTo("flat_file_name") == 0){
			this.flatFileName= args[1];
		}
		else if(args[0].compareTo("number_of_columns") == 0){
			this.numberOfColumns= Integer.parseInt(args[1]);
		}
		else if(args[0].compareTo("number_of_flat_files") == 0){
			this.numberOfFlatFiles = Integer.parseInt(args[1]);
		}
		else if(args[0].compareTo("column_separator") == 0){
			this.columnSeparator= args[1];
		}
		else if(args[0].compareTo("row_separator") == 0){
			this.rowSeparator= args[1];
		}
		else if(args[0].compareTo("text_qualifier") == 0){
			this.textQualifier = args[1];
		}	
		else if(args[0].compareTo("log_folder") == 0){
			this.logFolder = args[1];
		}		
	}
}