import java.io.*;

public class FlatFile{
	
	private File inputFile;
	private BufferedReader inputStream;
	private String error;
	private int numberOfExceptions;
	
	public String rowSeparator;
	public String columnDelimiter;
	public int numberOfColumns;
	public String textQualifier;
	public String fileName;
	
	
	
	public FlatFile(String flatFileName, String rowSeparator, String columnDelimiter, 
			int numberOfColumns, String textQualifier){
		this.fileName = flatFileName;
		this.rowSeparator = fixRowSeparator(rowSeparator); 
		this.columnDelimiter = columnDelimiter;
		this.numberOfColumns = numberOfColumns;
		this.textQualifier = textQualifier;
		this.error = "";
	}

	public String fixRowSeparator(String argument){
		String result = argument;
		if(argument.compareTo("\\n") == 0)
			result = "\n";		
		else if(argument.compareTo("\\r\\n") == 0)
			result = "\r\n";
		return result;
	}
	
	private void exceptionTreatment(Exception e){
		String lineBreak = System.getProperty("line.separator");
		numberOfExceptions++;
		System.err.println(e);
		error += "Exception! "+lineBreak+e.toString()+lineBreak+lineBreak; 
	}
	
	public int getNumberOfExceptions(){
		return numberOfExceptions;
	}
	

	public boolean openFile(){
		boolean result = true;
		try{
			inputFile = new File(fileName);
			inputStream = new BufferedReader(new FileReader(inputFile));
		}
		catch(FileNotFoundException e){
			exceptionTreatment(e);
			result = false;
		}
		return result;
	}
	
	private boolean isEndOfLine(String string){
		boolean result = false;
		if(string != null && string.endsWith(rowSeparator))			
			result = true;			
		return result;		
	}
	
	public String getLine(){
		StringBuilder line = new StringBuilder();
		int temp;
		char c;
		try{
			while(!isEndOfLine(line.toString()) && ((temp = inputStream. read()) != -1)){
				c = (char)temp;
				line.append(c);
			}		
		} catch (IOException e) {
			exceptionTreatment(e);
		}
		return line.toString().replace(rowSeparator, "");
	}
	
	public boolean closeFile(){
		boolean result = true;
		try{
			inputStream.close();	
		} catch (IOException e) {
			exceptionTreatment(e);
			result = false;
		}
		return result;
	}
	
	public String getErrors(){
		return error;
	}
	
}
