import java.io.*;
import java.util.Calendar;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Log {

	protected String logFileName;
	private String flatFileName;
	protected File outputFile;
	protected BufferedWriter outputStream;
	
	protected int numberOfWarnings;
	
	public Log(String logFileName, String flatFileName){
		this.logFileName = logFileName;
		this.flatFileName = flatFileName;
		this.numberOfWarnings= 0;
	}
	
	public Log(String flatFileName){
		this.logFileName = flatFileName+"_LOG";
		this.flatFileName = flatFileName;
		this.numberOfWarnings= 0;		
	}
	
	public boolean openFile(){
		boolean result = true;
		try{
			outputFile = new File(logFileName);
			outputStream = new BufferedWriter(new FileWriter(outputFile));
		}
		catch(FileNotFoundException e){
			e.printStackTrace();
			System.err.println(e);
			result = false;
		} catch (IOException e) {
			e.printStackTrace();
			System.err.println(e);
			result = false;
		}
		return result;
	}
	
	public void closeFile(){
		try{
			outputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
			System.err.println(e);
		}
	}
	
	public String errorTable(int errorCode){
		String error;
		
		switch(errorCode){
			case 1: 
				error = "Wrong Number of Columns";
				break;
			case 2: 
				error = "Wrong Text Qualifier";
				break;
			case 3:
				error = "Wrong Number of FlatFiles";
				break;
			default: 
				error = "Unknown Error";
		}
		
		return error;
	}
	
	public void writeHeader(String columnDelimiter, int numberOfColumns, String textQualifier){
		Date date = Calendar.getInstance().getTime();
		String lb = System.getProperty("line.separator");
		try {
			outputStream.write("Flat File: "+flatFileName+lb+"Date: "+date.toString()+lb+lb);
			outputStream.write("Column Separator: "+columnDelimiter+lb+"Number of Columns: "+ numberOfColumns+lb+"Text Qualifier: "+textQualifier+lb+lb);
		} catch (IOException e) {
			e.printStackTrace();
			System.err.println(e);
		}
	}
	
	public void write(int errorCode){
		String error = errorTable(errorCode);
		String lb = System.getProperty("line.separator");
		
		numberOfWarnings++;		
		try{
			outputStream.write("Warning!"+lb+"Error: "+error+lb+lb);
		}
		catch(IOException e){
			e.printStackTrace();
			System.err.println(e);
		}
	}
	
	
	public void write(int errorCode, int rowNumber){
		String error = errorTable(errorCode);
		String lb = System.getProperty("line.separator");
		
		numberOfWarnings++;		
		try{
			outputStream.write("Warning!"+lb+"Error: "+error+lb+"Row: "+rowNumber+lb+lb);
		}
		catch(IOException e){
			e.printStackTrace();
			System.err.println(e);
		}
	}
	
	public void write(String exceptions){		
		try{
			outputStream.write(exceptions);
		}
		catch(IOException e){
			e.printStackTrace();
			System.err.println(e);
		}
	}
	
	public void writeFooter(int numberOfExceptions){		
			try {
				outputStream.write("Total of Warnings: "+numberOfWarnings+"\n");
				outputStream.write("Total of Exceptions: "+numberOfExceptions+"\n");
			} catch (IOException e) {
				e.printStackTrace();
				System.err.println(e);
			}		
	}
	
	public int getNumberOfWarnings(){
		return numberOfWarnings;		
	}


}

